# -*- coding:utf-8 -*-

from datetime import date
from dateutil.relativedelta import relativedelta
from odoo import api, fields, models
from odoo.osv import expression


class FleetVehicle(models.Model):
    _name = "fleet.vehicle"
    _inherit = ['fleet.vehicle']

    driver_corporate_email = fields.Char('Corporate Email', compute='_compute_driver_related', store=False,
                                         search='_search_driver_corporate_email')
    driver_work_location = fields.Char('Work Location', compute='_compute_driver_related', store=False)

    @api.depends('co2_fee', 'log_contracts', 'log_contracts.state', 'log_contracts.cost_generated')
    def _compute_total_cost(self):
        super(FleetVehicle, self)._compute_total_cost()
        for car in self:
            contracts_8_percent = car.log_contracts.filtered(
                lambda contract: contract.state == 'open' and contract.cost_frequency == 'internal_8percent')
            for contract in contracts_8_percent:
                car.total_cost += contract.cost_generated

    @api.depends('driver_id')
    def _compute_driver_related(self):
        # Get the employees whose private partner is a driver of a vehicle in self
        employee_domain = [('work_contact_id', 'in', self.driver_id.ids)]
        drivers_employees = self.env['hr.employee'].sudo().with_context(active_test=False).search_read(employee_domain, ['work_contact_id', 'user_id', 'work_location_id'])

        # Get the email and work location of the users related to these employees
        user_domain = [('id', 'in', [emp['user_id'][0] for emp in drivers_employees if emp.get('user_id')])]
        users_data = self.env['res.users'].sudo().with_context(active_test=False).search_read(user_domain, ['email'])
        users_map = {user['id']: user for user in users_data}

        # Reorganize everything in an easy-to-query {'driver_id' => {'email', 'work_location'}} mapping
        user_info_by_driver_id = {
            emp['work_contact_id'][0]: [users_map[emp['user_id'][0]], (emp.get('work_location_id') or [False, False])[1]] for emp in drivers_employees if emp.get('work_contact_id') and emp.get('user_id')
        }

        # Assignment time!
        for vehicle in self:
            user_info = user_info_by_driver_id.get(vehicle.driver_id.id, {}) or [False, False]
            vehicle.driver_corporate_email = (user_info[0] or {}).get('email', False)
            vehicle.driver_work_location = user_info[1]

    @api.model
    def _search_driver_corporate_email(self, operator, value):
        emp_domain = [('user_id.email', 'ilike', value), ('work_contact_id', '!=', False)]
        res = self.env['hr.employee'].search_read(emp_domain, ['work_contact_id'])
        partner_ids = [emp['work_contact_id'][0] for emp in res]
        return [('driver_id', 'in', partner_ids)]

    @api.model_create_multi
    def create(self, vals_list):
        vehicles = super(FleetVehicle, self).create(vals_list)
        contracts_vals_list = []
        for vehicle in vehicles:
            # copy attachments from the vehicule model
            attachments = self.env['ir.attachment'].search([
                ('res_id', '=', vehicle.model_id.id),
                ('res_model', '=', vehicle.model_id._name)])
            for attachment in attachments:
                attachment.copy({'res_model': vehicle._name, 'res_id': vehicle.id})

            if not vehicle.log_contracts:
                cost_frequency = 'monthly'
                start_date = fields.Date.context_today(self).replace(day=1)
                expiration_date = start_date + relativedelta(years=30, days=-1)
                if vehicle.vehicle_type == 'car':
                    cost_frequency = 'internal_8percent'
                if vehicle.vehicle_type == 'bike':
                    expiration_date = start_date + relativedelta(years=3, days=-1)
                if vehicle.vehicle_type == 'terminal':
                    expiration_date = start_date + relativedelta(years=2, days=-1)
                contracts_vals_list.append({
                    'vehicle_id': vehicle.id,
                    'cost_generated': vehicle.model_id.default_recurring_cost_amount_depreciated,
                    'recurring_cost_amount_depreciated': vehicle.model_id.default_recurring_cost_amount_depreciated,
                    'purchaser_id': vehicle.driver_id.id,
                    'company_id': vehicle.company_id.id,
                    'user_id': vehicle.manager_id.id if vehicle.manager_id else self.env.user.id,
                    'cost_frequency': cost_frequency,
                    'start_date': start_date,
                    'expiration_date': expiration_date,
                })
        self.env['fleet.vehicle.log.contract'].create(contracts_vals_list)
        return vehicles

    def _get_co2_fee(self, co2, fuel_type):
        if self.vehicle_type == 'terminal':
            return 0
        return super(FleetVehicle, self)._get_co2_fee(co2, fuel_type)

class FleetVehicleModel(models.Model):
    _inherit = 'fleet.vehicle.model'

    vehicle_type = fields.Selection(selection_add=[('terminal', 'Electrical Terminal')], ondelete={'terminal': 'set default'})


class FleetVehicleLogContract(models.Model):
    _inherit = 'fleet.vehicle.log.contract'

    def _get_default_expiration_date(self):
        return date.today() + relativedelta(years=+5, days=-1)

    cost_frequency = fields.Selection(
        selection_add=[("internal_8percent", "8% Depreciation (Internal)")],
        ondelete={'internal_8percent': 'set default'})
    start_date = fields.Date(default=fields.Date.today)
    expiration_date = fields.Date(default=_get_default_expiration_date)

    @api.model
    def default_get(self, default_fields):
        res = super(FleetVehicleLogContract, self).default_get(default_fields)
        contract = self.env.ref('hr_contract_salary_internal.fleet_service_type_odoo_sa', raise_if_not_found=False)
        res['cost_subtype_id'] = contract and contract.id or False
        return res

    def update_recurring_cost_amount_depreciated(self):
        domain = [
            ('state', '=', 'open'),
            ('cost_frequency', '=', 'internal_8percent'),
            ('start_date', '!=', False),
            ('cost_generated', '!=', False)]
        if self.env.context.get('active_ids', False):
            domain += [('id', 'in', self.env.context.get('active_ids'))]
        contracts = self.search(domain)

        today = date.today()
        for contract in contracts:
            start_date = fields.Date.from_string(contract.vehicle_id.first_contract_date or contract.start_date)
            age_delta = relativedelta(today, start_date)
            age = age_delta.years * 12 + age_delta.months
            # 8 percents of depreciation
            if age >= 0 and age < 7 * 12:
                recurring_cost_amount_depreciated = contract.cost_generated * pow(0.92 ** (1.0/12.0), age)
            # Maintenance costs only
            else:
                recurring_cost_amount_depreciated = 175.0
            contract.recurring_cost_amount_depreciated = recurring_cost_amount_depreciated
